<?php

namespace Drupal\Tests\posthog_cookies\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\cookies\Traits\CookiesCacheClearTrait;

/**
 * Tests posthog_cookies Javascript related functionalities.
 *
 * @group posthog_cookies
 */
class PosthogCookiesFunctionalJavascriptTest extends WebDriverTestBase {
  use CookiesCacheClearTrait;

  /**
   * The user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $user;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'test_page_test',
    'filter_test',
    'cookies',
    'posthog_js',
    'posthog_cookies',
  ];

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->config('system.site')->set('page.front', '/test-page')->save();
    $this->user = $this->drupalCreateUser([]);
    $this->drupalPlaceBlock('cookies_ui_block');
    // Set google_analytics settings:
    $this->config('posthog.settings')->set('host', 'https://eu.i.posthog.com')->save();
    $this->config('posthog.settings')->set('api_key', 'phc_XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')->save();
    // Test that scripts are knocked out even when JS aggregation is enabled.
    $this->config('system.performance')->set('js.preprocess', TRUE)->save();
    $this->clearBackendCaches();
  }

  /**
   * Tests if the cookies ga javascript file is correctly knocked in / out.
   */
  public function testGoogleAnalyticsJsCorrectlyKnocked() {
    $session = $this->assertSession();
    $driver = $this->getSession()->getDriver();

    $this->drupalGet('<front>');

    // Check that all required posthog javascript files are present:
    $session->elementsCount('css', 'script[src*="js/posthog_cookies.js"]', 1);
    $session->elementsCount('css', 'script[src*="https://eu-assets.i.posthog.com"]', 1);
    $session->elementsCount('css', 'script#posthog-js-cdn', 1);
    $session->elementsCount('css', 'script#posthog-js-init', 1);

    // Check, that the posthog cookie is not set:
    $this->assertEmpty($driver->getCookie('ph_phc_XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'));

    // Fire consent script, accept all cookies:
    $script = "var options = { all: true };
        document.dispatchEvent(new CustomEvent('cookiesjsrSetService', { detail: options }));";
    $this->getSession()->getDriver()->executeScript($script);

    $this->drupalGet('<front>');

    // Check that all required posthog javascript files are still present:
    $session->elementsCount('css', 'script[src*="js/posthog_cookies.js"]', 1);
    $session->elementsCount('css', 'script[src*="https://eu-assets.i.posthog.com"]', 1);
    $session->elementsCount('css', 'script#posthog-js-cdn', 1);
    $session->elementsCount('css', 'script#posthog-js-init', 1);

    // Check, that the posthog cookie is set now:
    $this->assertNotEmpty($driver->getCookie('ph_phc_XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'));
  }

  /**
   * Tests if the js file is correctly knocked in / out with js aggregation on.
   */
  public function testGoogleAnalyticsJsCorrectlyKnockedWithJsAggregation() {
    // Test that scripts are knocked out even when JS aggregation is enabled.
    $this->config('system.performance')->set('js.preprocess', TRUE)->save();

    $session = $this->assertSession();
    $driver = $this->getSession()->getDriver();

    $this->drupalGet('<front>');

    // Check that all required posthog javascript files are present:
    $session->elementsCount('css', 'script[src*="js/posthog_cookies.js"]', 1);
    $session->elementsCount('css', 'script[src*="https://eu-assets.i.posthog.com"]', 1);
    $session->elementsCount('css', 'script#posthog-js-cdn', 1);
    $session->elementsCount('css', 'script#posthog-js-init', 1);

    // Check, that the posthog cookie is not set:
    $this->assertEmpty($driver->getCookie('ph_phc_XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'));

    // Fire consent script, accept all cookies:
    $script = "var options = { all: true };
        document.dispatchEvent(new CustomEvent('cookiesjsrSetService', { detail: options }));";
    $this->getSession()->getDriver()->executeScript($script);

    $this->drupalGet('<front>');

    // Check that all required posthog javascript files are still present:
    $session->elementsCount('css', 'script[src*="js/posthog_cookies.js"]', 1);
    $session->elementsCount('css', 'script[src*="https://eu-assets.i.posthog.com"]', 1);
    $session->elementsCount('css', 'script#posthog-js-cdn', 1);
    $session->elementsCount('css', 'script#posthog-js-init', 1);

    // Check, that the posthog cookie is set now:
    $this->assertNotEmpty($driver->getCookie('ph_phc_XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'));
  }

}
