<?php

declare(strict_types=1);

namespace Drupal\posthog_js\Form;

use Drupal\Core\Condition\ConditionInterface;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Condition\ConditionPluginCollection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Tab Title Attention settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * Constructs a \Drupal\posthog_js\Form\SettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $conditionManager
   *   The condition manager.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, protected ConditionManager $conditionManager, protected TypedConfigManagerInterface $typedConfigManager) {
    parent::__construct($config_factory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('config.factory'),
      $container->get('plugin.manager.condition'),
      $container->get('config.typed'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'posthog_js_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['posthog_js.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#description' => $this->t("Check to add the JavaScript tracking code to matching pages."),
      '#default_value' => $this->config('posthog_js.settings')->get('enabled'),
    ];

    $form['cdn'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Load from Posthog ("CDN" [recommended])'),
      '#description' => $this->t("Check to load Posthog JavaScript from Posthog (recommended).<br />If disabled, you must provide the (posthog-js) library yourself, for example using <code>composer require npm-asset/posthog-js</code>.<br /><a href='https://posthog.com/docs/privacy/gdpr-compliance' target='_blank'></a>Note that Posthog points out its GDPR compliance.</a>"),
      '#default_value' => $this->config('posthog_js.settings')->get('cdn'),
    ];

    $accessJsSettings = $this->currentUser()->hasPermission('administer posthog_js settings javascript');
    $accessJsSettingsWarning = !$accessJsSettings ? ' <em>' . $this->t('This field has been disabled because you do not have the "administer posthog_js settings javascript" permission, which is required here.') . '</em>' : '';
    $form['init_config_json'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Initialization configuration (JSON)'),
      '#description' => $this->t("Optional additional Posthog initialization JSON code. Add values to override <a href=\"https://posthog.com/docs/libraries/js#config\" target=\"_blank\">the Posthog defaults</a>. Leave empty to use the defaults.") . $accessJsSettingsWarning,
      '#default_value' => $this->config('posthog_js.settings')->get('init_config_json') ?? "{}",
      '#placeholder' => "{\n  // autocapture: true,\n  // capture_pageview: true,\n}",
      '#disabled' => !$accessJsSettings,
    ];

    $form['conditions'] = [
      '#tree' => TRUE,
    ];
    $form['conditions'] += $this->buildConditions([], $form_state);

    return parent::buildForm($form, $form_state);
  }

  /**
   * Helper function for building the conditions form elements.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form array with the visibility UI added in.
   */
  protected function buildConditions(array $form, FormStateInterface $form_state) {
    $form['conditions_tabs'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Conditions'),
      '#descriptions' => $this->t('Define where Posthog JS should be loaded / initialized.'),
      '#parents' => ['conditions_tabs'],
      '#attached' => [
        'library' => [
          // @todo Is this needed??
          'block/drupal.block',
        ],
      ],
      '#tree' => TRUE,
    ];

    $conditions = $this->config('posthog_js.settings')->get('conditions');
    foreach ($this->conditionManager->getDefinitions() as $condition_id => $definition) {
      /** @var \Drupal\Core\Condition\ConditionInterface $condition */
      $condition = $this->conditionManager->createInstance($condition_id, $conditions[$condition_id] ?? []);
      $form_state->set(['conditions', $condition_id], $condition);
      $condition_form = $condition->buildConfigurationForm([], $form_state);
      $condition_form['#type'] = 'details';
      $condition_form['#title'] = $condition->getPluginDefinition()['label'];
      $condition_form['#group'] = 'conditions_tabs';
      $form[$condition_id] = $condition_form;
    }

    if (isset($form['node_type'])) {
      $form['node_type']['#title'] = $this->t('Content types');
      $form['node_type']['bundles']['#title'] = $this->t('Content types');
      $form['node_type']['negate']['#type'] = 'value';
      $form['node_type']['negate']['#title_display'] = 'invisible';
      $form['node_type']['negate']['#value'] = $form['node_type']['negate']['#default_value'];
    }
    if (isset($form['user_role'])) {
      $form['user_role']['#title'] = $this->t('Roles');
      unset($form['user_role']['roles']['#description']);
      $form['user_role']['negate']['#type'] = 'value';
      $form['user_role']['negate']['#value'] = $form['user_role']['negate']['#default_value'];
    }
    if (isset($form['request_path'])) {
      $form['request_path']['#title'] = $this->t('Pages');
      $form['request_path']['negate']['#type'] = 'radios';
      $form['request_path']['negate']['#default_value'] = (int) $form['request_path']['negate']['#default_value'];
      $form['request_path']['negate']['#title_display'] = 'invisible';
      $form['request_path']['negate']['#options'] = [
        $this->t('Show for the listed pages'),
        $this->t('Hide for the listed pages'),
      ];
    }
    if (isset($form['language'])) {
      $form['language']['negate']['#type'] = 'value';
      $form['language']['negate']['#value'] = $form['language']['negate']['#default_value'];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    try {
      // Validate JSON:
      $initConfigJson = trim($form_state->getValue('init_config_json') ?? '');
      if (!empty($initConfigJson)) {
        $form_state->setValue('init_config_json', json_encode(json_decode($initConfigJson ?: '{}', FALSE, 5, JSON_THROW_ON_ERROR | JSON_INVALID_UTF8_SUBSTITUTE), JSON_FORCE_OBJECT | JSON_INVALID_UTF8_SUBSTITUTE | JSON_PRETTY_PRINT | JSON_THROW_ON_ERROR));
      }
    }
    catch (\JsonException $e) {
      $form_state->setErrorByName('init_config_json', $this->t('Invalid JSON'));
    }

    $this->validateConditions($form, $form_state);
  }

  /**
   * Helper function to independently validate the visibility UI.
   *
   * @param array $form
   *   A nested array form elements comprising the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  protected function validateConditions(array $form, FormStateInterface $form_state) {
    // Validate visibility condition settings.
    foreach ($form_state->getValue('conditions') as $condition_id => $values) {
      // Allow the condition to validate the form.
      $condition = $form_state->get(['conditions', $condition_id]);
      assert($condition instanceof ConditionInterface);
      $condition->validateConfigurationForm($form['conditions'][$condition_id], SubformState::createForSubform($form['conditions'][$condition_id], $form, $form_state));
    }
  }

  /**
   * Helper function to retrieve the conditions configuration form values.
   *
   * @param array $form
   *   A nested array form elements comprising the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  protected function getConditionsConfigurationValue(array $form, FormStateInterface $form_state) {
    $conditionsCollection = new ConditionPluginCollection($this->conditionManager, []);
    foreach ($form_state->getValue('conditions') as $condition_id => $values) {
      // Allow the condition to submit the form.
      $condition = $form_state->get(['conditions', $condition_id]);
      assert($condition instanceof ConditionInterface);
      $condition->submitConfigurationForm($form['conditions'][$condition_id], SubformState::createForSubform($form['conditions'][$condition_id], $form, $form_state));

      $condition_configuration = $condition->getConfiguration();
      $conditionsCollection->addInstanceId($condition_id, $condition_configuration);
    }
    return $conditionsCollection->getConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $initConfigJson = $form_state->getValue('init_config_json');
    $this->config('posthog_js.settings')
      ->set('enabled', $form_state->getValue('enabled'))
      ->set('cdn', $form_state->getValue('cdn'))
      ->set('init_config_json', $initConfigJson)
      ->set('conditions', $this->getConditionsConfigurationValue($form, $form_state))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
