<?php

declare(strict_types=1);

namespace Drupal\posthog_php;

use Drupal\Core\Config\ConfigFactoryInterface;
use PostHog\PostHog;

/**
 * Mirrors the PostHog class from PostHog SDK.
 */
class Sdk implements SdkInterface {

  /**
   * Constructs a Sdk object.
   */
  public function __construct(
    private readonly ConfigFactoryInterface $configFactory,
    protected $initalized = FALSE,
  ) {
    $this->init();
  }

  /**
   * {@inheritDoc}
   */
  public static function getVersion(): string {
    return PostHog::VERSION;
  }

  /**
   * {@inheritdoc}
   */
  public function init(): void {
    if ($this->initalized) {
      return;
    }
    $apiKey = $this->configFactory->get('posthog.settings')->get('api_key');
    $host = $this->configFactory->get('posthog.settings')->get('host');
    PostHog::init($apiKey,
      ['host' => $host]
    );
    $this->initalized = TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function identify(array $message): bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::identify($message);
  }

  /**
   * {@inheritdoc}
   */
  public function groupIdentify(array $message): bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::groupIdentify($message);
  }

  /**
   * {@inheritdoc}
   */
  public function isFeatureEnabled(string $key, string $distinctId, array $groups = [], array $personProperties = [], array $groupProperties = [], bool $onlyEvaluateLocally = FALSE, bool $sendFeatureFlagEvents = TRUE): ?bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::isFeatureEnabled($key, $distinctId, $groups, $personProperties, $groupProperties, $onlyEvaluateLocally, $sendFeatureFlagEvents);
  }

  /**
   * {@inheritdoc}
   */
  public function getFeatureFlag(string $key, string $distinctId, array $groups = [], array $personProperties = [], array $groupProperties = [], bool $onlyEvaluateLocally = FALSE, bool $sendFeatureFlagEvents = TRUE): null|bool|string {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::getFeatureFlag($key, $distinctId, $groups, $personProperties, $groupProperties, $onlyEvaluateLocally, $sendFeatureFlagEvents);
  }

  /**
   * {@inheritdoc}
   */
  public function getAllFlags(string $distinctId, array $groups = [], array $personProperties = [], array $groupProperties = [], bool $onlyEvaluateLocally = FALSE): array {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::getAllFlags($distinctId, $groups, $personProperties, $groupProperties, $onlyEvaluateLocally);
  }

  /**
   * {@inheritdoc}
   */
  public function fetchFeatureVariants(string $distinctId, array $groups = []): array {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::fetchFeatureVariants($distinctId, $groups);
  }

  /**
   * {@inheritdoc}
   */
  public function alias(array $message): bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::alias($message);
  }

  /**
   * {@inheritDoc}
   */
  public function aliasUser(string $distinctId, string $alias): bool {
    return $this->alias([
      'distinctId' => $distinctId,
      'alias' => $alias,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function raw(array $message): bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::raw($message);
  }

  /**
   * {@inheritdoc}
   */
  public function validate($msg, $type): bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::validate($msg, $type);
  }

  /**
   * {@inheritdoc}
   */
  public function flush(): bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::flush();
  }

  /**
   * {@inheritDoc}
   */
  public function capture(array $message): bool {
    if (!$this->initalized) {
      $this->init();
    }
    return PostHog::capture($message);
  }

  /**
   * {@inheritDoc}
   */
  public function captureUserEvent(string $distinctId, string $event, array $properties = []): bool {
    $capture = [
      'distinctId' => $distinctId,
      'event' => $event,
    ];
    if (!empty($properties)) {
      $capture['properties'] = $properties;
    }

    $this->capture($capture);
  }

  /**
   * {@inheritDoc}
   */
  public function captureUserPageView(string $distinctId, string $currentUrl): bool {
    $this->captureUserEvent($distinctId, SdkInterface::CAPTURE_EVENT_PAGEVIEW, ['$current_url' => $currentUrl]);
  }

}
