<?php

declare(strict_types=1);

namespace Drupal\posthog_php;

/**
 * Mirror of the PostHog PHP SDK.
 *
 * Mirrors https://github.com/PostHog/posthog-php/blob/master/lib/PostHog.php
 * as interface.
 *
 * All calls are initialized.
 *
 * Last updated on version 3.0.3.
 *
 * @see https://posthog.com/docs/libraries/php
 */
interface SdkInterface {

  const CAPTURE_EVENT_PAGEVIEW = '$pageview';

  /**
   * Returns the current PostHog SDK version.
   *
   * E.g. 3.0.3
   *
   * @return string
   */
  public static function getVersion(): string;

  /**
   * Initializes the default client to use.
   *
   * @throws Exception
   */
  public function init(): void;

  /**
   * Captures a user action.
   *
   * Expects a message as array, for example:
   * [
   *  'distinctId' => 'distinct_id_of_the_user',
   *   'event' => 'user_signed_up'
   * ]
   *
   * @param array $message
   *
   * @return bool
   *   Whether the capture call succeeded
   *
   * @throws Exception.
   *
   * @see https://posthog.com/docs/libraries/php#capturing-events
   */
  public function capture(array $message): bool;

  /**
   * Helper to capture an event on top of ::capture().
   *
   * @param string $distinctId
   *   The distinct user id.
   * @param string $event
   *   The event id.
   * @param array $properties
   *   Optional additional properties array, e.g.
   *   [
   *     'login_type' => 'email',
   *     'is_free_trial' => 'true',
   *   ]
   *     or
   *   [
   *     '$current_url' => 'https://example.com'
   *   ].
   *
   * @return bool
   *   Whether the capture call succeeded
   *
   * @see https://posthog.com/docs/libraries/php#capturing-events
   */
  public function captureUserEvent(string $distinctId, string $event, array $properties = []): bool;

  /**
   * Helper to capture page view events on top of ::capture().
   *
   * @param string $distinctId
   *   The distinct user id.
   * @param string $currentUrl
   *   The current URL.
   *
   * @return bool
   *   Whether the capture call succeeded
   */
  public function captureUserPageView(string $distinctId, string $currentUrl): bool;

  /**
   * Tags properties about the user.
   *
   * @param array $message
   *
   * @return boolean whether the identify call succeeded
   *
   * @throws Exception.
   */
  public function identify(array $message): bool;

  /**
   * Adds properties to a group.
   *
   * @param array $message
   *   Must contain keys `groupType`, `groupKey`, `properties`.
   *
   * @return boolean whether the groupIdentify call succeeded
   *
   * @throws Exception.
   */
  public function groupIdentify(array $message): bool;

  /**
   * Tells if the feature flag is enabled for this distinct id.
   *
   * @param string $key
   * @param string $distinctId
   * @param array $groups
   * @param array $personProperties
   * @param array $groupProperties
   *
   * @return bool
   *
   * @throws Exception.
   */
  public function isFeatureEnabled(
    string $key,
    string $distinctId,
    array $groups = [],
    array $personProperties = [],
    array $groupProperties = [],
    bool $onlyEvaluateLocally = FALSE,
    bool $sendFeatureFlagEvents = TRUE,
  ): NULL | bool;

  /**
   * Get the feature flag value for this distinct id.
   *
   * @param string $key
   * @param string $distinctId
   * @param array $groups
   * @param array $personProperties
   * @param array $groupProperties
   *
   * @return boolean | string
   *
   * @throws Exception.
   */
  public function getFeatureFlag(
    string $key,
    string $distinctId,
    array $groups = [],
    array $personProperties = [],
    array $groupProperties = [],
    bool $onlyEvaluateLocally = FALSE,
    bool $sendFeatureFlagEvents = TRUE,
  ): null | bool | string;

  /**
   * Get all enabled flags for distinct_id.
   *
   * @param string $distinctId
   * @param array $groups
   * @param array $personProperties
   * @param array $groupProperties
   *
   * @return array
   *
   * @throws Exception
   */
  public function getAllFlags(
    string $distinctId,
    array $groups = [],
    array $personProperties = [],
    array $groupProperties = [],
    bool $onlyEvaluateLocally = FALSE,
  ): array;

  /**
   * Fetch the feature variants.
   *
   * @param string $distinctId
   *
   * @return array
   *
   * @throws Exception
   */
  public function fetchFeatureVariants(string $distinctId, array $groups = []): array;

  /**
   * Aliases the distinct id from a temporary id to a permanent one.
   *
   * @param array $message
   *   distinct id to alias from.
   *
   * @return boolean whether the alias call succeeded
   *
   * @throws Exception.
   */
  public function alias(array $message): bool;

  /**
   * Helper method based on ::alias() to alias a certain user.
   *
   * @param string $distinctId
   * @param string $alias
   * @return bool
   */
  public function aliasUser(string $distinctId, string $alias): bool

  /**
   * Send a raw (prepared) message.
   *
   * @param array $message
   *   distinct id to alias from.
   *
   * @return boolean whether the alias call succeeded.
   */
  public function raw(array $message): bool;

  /**
   * Validate common properties.
   *
   * @param array $msg
   * @param string $type
   *
   * @throws Exception
   */
  public function validate($msg, $type): bool;

  /**
   * Flush the client.
   */
  public function flush(): bool;

}
